# ifndef _TIMER_H
# define _TIMER_H

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>

class timer {
  
  double m_start_cpu;             // msec cpu
  double m_last_cpu;              // msec cpu
  inline double get_cpu() {       // msec cpu
    rusage usage;
    getrusage(RUSAGE_SELF,&usage);
    double cpu = usage.ru_utime.tv_sec*1000.0 + usage.ru_utime.tv_usec/1000.0;
    if (cpu < 0.0) cpu = 0.0;
    return cpu;
  };

  inline timer(const timer&);
  inline timer& operator=(const timer&);
public:
  /*-desc------
    defines and initializes a timer for measuring cpu-time;
  */
  inline timer () : m_start_cpu(get_cpu()),
                    m_last_cpu(m_start_cpu) {
  }
  
  /*-desc------
    returns the cpu-time used since the last call to used_time()
    or since the timer was created.
    If \Imsg is given, the cpu-time used is output to \If,
    preceded by the message \Imsg.
  */
  inline double used_time(const char* msg, FILE* f = stdout) {
    // time since last call in seconds
    double now  = get_cpu();
    double diff = (now - m_last_cpu)*0.001;
    // printf("last %.12f now %.12f diff %.12f\n",m_last_cpu,now, diff);
    m_last_cpu  = now;
    if (diff < 0.0) diff = 0.0;
    if (msg) fprintf(f,"%s %.3f\n",msg,diff);
    return diff;
  }

}; // timer

#endif
